/* $Header */

#include <windows.h>
#include "exports.h"
#include "cwstart.h"


/* flags respected by Grep, from util.c	*/
#define	GREP_FLAGS (SEARCH_REGEX | SEARCH_MAXIMAL | SEARCH_IGCASE)

LIBMAIN    /* defines the LibMain() function for Borland C or WIN32 */

void DLL
_init()
{
	LibExport("long FGrepQuick LPSTR WORD");
	LibExport("long GrepQuick WORD");
	LibExport("long FGrepPrompt LPSTR WORD");
	LibExport("long GrepPrompt WORD");
}

/*
 * Front end for FGrep that uses the word under the cursor as
 * the search pattern.  FGrep gets called through LibFunctionExec
 * because that's the only way to take advantage of defaults,
 * since you can't query the fgrep stuff for flags and filespec.
 */

long DLL
FGrepQuick(LPSTR filespec, WORD flags)
{
	int argc;
	long retval;
	LPSTR args;
	LPSTR curWord;

	argc = ArgCount();
	MarkSavePos();

	curWord = _ReadWord();
	args = MemAlloc(1024);
	if (argc >= 2)
		wsprintf(args, "FGrep %s %s %d", curWord, filespec, (int)flags);
	else if (argc == 1)
		wsprintf(args, "FGrep %s %s", curWord, filespec);
	else
		wsprintf(args, "FGrep %s", curWord);
	retval = (long)LibFunctionExec(args);
	MemFree(args);
	StrFree(curWord);
	MarkRestorePos();
	return(retval);
}

/*
 * This front end for FGrep prompts the user for the string to search
 * for.  The word under the cursor is used as the default prompt.
 */

long DLL
FGrepPrompt(LPSTR filespec, WORD flags)
{
	int argc;
	long retval;
	LPSTR args;
	LPSTR curWord;
	LPSTR target;

	argc = ArgCount();
	MarkSavePos();

	curWord = _ReadWord();
	target = MsgPromptHistory("XGREPPROMPT", "String to grep for:", curWord);
	if (target)
	{
		args = MemAlloc(1024);
		if (argc >= 2)
			wsprintf(args, "FGrep %s %s %d", target, filespec, (int)flags);
		else if (argc == 1)
			wsprintf(args, "FGrep %s %s", target, filespec);
		else
			wsprintf(args, "FGrep %s", target);
		retval = (long)LibFunctionExec(args);
		MemFree(args);
	}
	StrFree(curWord);
	StrFree(target);
	MarkRestorePos();
	return(retval);
}

/*
 * This version of Grep reads the word under the cursor and goes.
 * If search flags are supplied, they will be observed.  Otherwise,
 * search defaults are used.
 */

long DLL
GrepQuick(DWORD flags)
{
	long retval;
	LPSTR args;
	LPSTR curWord;
	DWORD oldFlags = SrchQFlags();

	if (!ArgCount())
		flags = (oldFlags & GREP_FLAGS) | SEARCH_FORWARD;
	SrchSetFlags(flags);

	MarkSavePos();
	curWord = _ReadWord();
	args = StrAppend(StrNew("Grep "), curWord);
	retval = (long)LibFunctionExec(args);
	StrFree(args);
	StrFree(curWord);
	MarkRestorePos();
	SrchSetFlags(oldFlags);
	return(retval);
}

/*
 * This is the same as GrepQuik, except that the user gets prompted
 * (with the current word as the default) before the search proceeds.
 */

long DLL
GrepPrompt(DWORD flags)
{
	long retval;
	LPSTR args;
	LPSTR curWord;
	LPSTR target;
	DWORD oldFlags = SrchQFlags();

	if (!ArgCount())
		flags = (oldFlags & GREP_FLAGS) | SEARCH_FORWARD;
	SrchSetFlags(flags);

	MarkSavePos();
	curWord = _ReadWord();
	target = MsgPromptHistory("XGREPPROMPT", "String to grep for:", curWord);
	if (target)
	{
		args = StrAppend(StrNew("Grep "), target);
		retval = (long)LibFunctionExec(args);
	}
	StrFree(args);
	StrFree(target);
	StrFree(curWord);
	MarkRestorePos();
	SrchSetFlags(oldFlags);
	return(retval);
}


